"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_json_languageservice_1 = require("vscode-json-languageservice");
const schema_json_1 = __importDefault(require("./schema.json"));
const ts_json_schema_generator_1 = require("ts-json-schema-generator");
// Modify of https://github.com/volarjs/services/blob/master/packages/json/src/index.ts
exports.default = (config = {}) => (context) => {
    // https://github.com/microsoft/vscode/blob/09850876e652688fb142e2e19fd00fd38c0bc4ba/extensions/json-language-features/server/src/jsonServer.ts#L150
    const triggerCharacters = ['"', ":"];
    if (!context) {
        return { triggerCharacters };
    }
    const jsonDocuments = new WeakMap();
    const jsonLs = (0, vscode_json_languageservice_1.getLanguageService)({});
    console.log('yue')
    try {
        const routeMetaSchema = (0, ts_json_schema_generator_1.createGenerator)(Object.assign({ skipTypeCheck: true, type: "RouteMeta" }, config)).createSchema("RouteMeta");
        schema_json_1.default.definitions = Object.assign(Object.assign({}, schema_json_1.default.definitions), routeMetaSchema.definitions);
    }
    catch (e) {
        console.log("[Error] volar-plugin-vue-router:");
        console.log(e);
    }
    jsonLs.configure({
        allowComments: false,
        schemas: [
            {
                fileMatch: ["*.customBlock_route_*.json"],
                uri: "foo://route-custom-block.schema.json",
                schema: schema_json_1.default,
            },
        ],
    });
    return {
        triggerCharacters,
        resolveRuleContext(context) {
            return __awaiter(this, void 0, void 0, function* () {
                yield worker(context.document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                    context.json = {
                        document: jsonDocument,
                        languageService: jsonLs,
                    };
                }));
                return context;
            });
        },
        provideCompletionItems(document, position) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.doComplete(document, position, jsonDocument);
            }));
        },
        resolveCompletionItem(item) {
            return jsonLs.doResolve(item);
        },
        provideDefinition(document, position) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.findDefinition(document, position, jsonDocument);
            }));
        },
        provideDiagnostics(document) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                const documentLanguageSettings = undefined; // await getSettings(); // TODO
                return (yield jsonLs.doValidation(document, jsonDocument, documentLanguageSettings, undefined // TODO
                ));
            }));
        },
        provideHover(document, position) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.doHover(document, position, jsonDocument);
            }));
        },
        provideDocumentLinks(document) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.findLinks(document, jsonDocument);
            }));
        },
        provideDocumentSymbols(document) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.findDocumentSymbols2(document, jsonDocument);
            }));
        },
        provideDocumentColors(document) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.findDocumentColors(document, jsonDocument);
            }));
        },
        provideColorPresentations(document, color, range) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.getColorPresentations(document, jsonDocument, color, range);
            }));
        },
        provideFoldingRanges(document) {
            return worker(document, () => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.getFoldingRanges(document);
            }));
        },
        provideSelectionRanges(document, positions) {
            return worker(document, (jsonDocument) => __awaiter(this, void 0, void 0, function* () {
                return yield jsonLs.getSelectionRanges(document, positions, jsonDocument);
            }));
        },
        provideDocumentFormattingEdits(document, range, options) {
            return worker(document, () => __awaiter(this, void 0, void 0, function* () {
                var _a, _b, _c;
                const options_2 = yield ((_b = (_a = context.env).getConfiguration) === null || _b === void 0 ? void 0 : _b.call(_a, "json.format"));
                if (!((_c = options_2 === null || options_2 === void 0 ? void 0 : options_2.enable) !== null && _c !== void 0 ? _c : true)) {
                    return;
                }
                return jsonLs.format(document, range, Object.assign(Object.assign({}, options_2), options));
            }));
        },
    };
    function worker(document, callback) {
        const jsonDocument = getJsonDocument(document);
        if (!jsonDocument)
            return;
        return callback(jsonDocument);
    }
    function getJsonDocument(textDocument) {
        if (textDocument.languageId !== "json" &&
            textDocument.languageId !== "jsonc")
            return;
        const cache = jsonDocuments.get(textDocument);
        if (cache) {
            const [cacheVersion, cacheDoc] = cache;
            if (cacheVersion === textDocument.version) {
                return cacheDoc;
            }
        }
        const doc = jsonLs.parseJSONDocument(textDocument);
        jsonDocuments.set(textDocument, [textDocument.version, doc]);
        return doc;
    }
};
//# sourceMappingURL=index.js.map